angular
	.module( "beamng.apps" )
	.directive( "evBattery", [ "StreamsManager", function ( StreamsManager ) {
		return {
			template:
				"<object class='bngApp' style='width:100%; height:100%; pointer-events: none' type='image/svg+xml' data='modules/apps/EvBatteryMonitor/ev-battery-monitor.svg?t=" + Date.now() + "'/>",
			replace: true,
			restrict: "EA",
			link: function ( scope, element, attrs ) {
				function clamp( val, min, max ) {
					return Math.max( Math.min( val, max ), min );
				}

				function maprange( val, srcMin, srcMax, dstMin, dstMax, clampOutput ) {
					if ( clampOutput !== false ) clampOutput = true;
					if ( typeof dstMin !== "number" ) dstMin = 0.0;
					if ( typeof dstMax !== "number" ) dstMax = 1.0;

					var normalized = ( val - srcMin ) / ( srcMax - srcMin ); // 0.0 to 1.0 when inside[ srcMin, srcMax ]

					var dstRange = dstMax - dstMin;
					var output = dstRange * normalized + dstMin;

					if ( clampOutput ) {
						var cMin = Math.min( dstMin, dstMax );
						var cMax = Math.max( dstMin, dstMax );

						output = clamp( output, cMin, cMax );
					}

					return output;
				}

				StreamsManager.add( [ "hybrid" ] );

				scope.$on( "$destroy", function () {
					StreamsManager.remove( [ "hybrid" ] );
				} );

				element.on( "load", function () {
					var svg = element[ 0 ].contentDocument;
					var rootLayer = svg.getElementById( "root-layer" );
					var batteryLevelGauge = {
						valOver: svg.getElementById( "val-bat-over" ),
						valUnder: svg.getElementById( "val-bat-under" ),
						clip: {
							rect: svg.getElementById( "clip-bat-rect" ),
							height: +svg.getElementById( "clip-bat-rect" ).getAttribute( "height" ),
							y: +svg.getElementById( "clip-bat-rect" ).getAttribute( "y" ),
						},
					};
					var powerDrawGauge = {
						valOver: svg.getElementById( "val-kw-over" ),
						valUnder: svg.getElementById( "val-kw-under" ),
						clip: {
							rect: svg.getElementById( "clip-kw-rect" ),
							height: +svg.getElementById( "clip-kw-rect" ).getAttribute( "height" ),
							y: +svg.getElementById( "clip-kw-rect" ).getAttribute( "y" ),
						},
					};

					const AverageWindow = 10;
					var stateOfChargeAvg = 0;
					var powerDrawAvg = 0;

					scope.$on( "streamsUpdate", function ( event, streams ) {
						var stateOfCharge = 0;
						var minStateOfCharge = 0;
						var maxStateOfCharge = 1;
						var powerDraw = 0;
						var maxPowerDraw = 1;

						if ( streams != null && streams.hybrid != null ) {
							rootLayer.style.opacity = 1.0;
							stateOfCharge = isNaN( streams.hybrid.soc ) ? 0.0 : streams.hybrid.soc;
							minStateOfCharge = isNaN( streams.hybrid.minSoc ) ? 0.0 : streams.hybrid.minSoc;
							maxStateOfCharge = isNaN( streams.hybrid.maxSoc ) ? 1.0 : streams.hybrid.maxSoc;
							powerDraw = isNaN( streams.hybrid.kw ) ? 0.0 : streams.hybrid.kw;
							maxPowerDraw = isNaN( streams.hybrid.maxKw ) ? 0.0 : streams.hybrid.maxKw;
						} else {
							rootLayer.style.opacity = 0.0;
						}

						// Calculate moving averages
						stateOfChargeAvg = stateOfChargeAvg + ( stateOfCharge - stateOfChargeAvg ) / AverageWindow;
						powerDrawAvg = powerDrawAvg + ( powerDraw - powerDrawAvg ) / AverageWindow;

						batteryLevelGauge.valOver.innerHTML = `${Math.round( stateOfChargeAvg * 100 )}%`;
						batteryLevelGauge.valUnder.innerHTML = batteryLevelGauge.valOver.innerHTML;
						powerDrawGauge.valOver.innerHTML = `${Math.round( powerDrawAvg )} kW`;
						powerDrawGauge.valUnder.innerHTML = powerDrawGauge.valOver.innerHTML;

						var normalizedStateOfCharge = maprange( stateOfChargeAvg, minStateOfCharge, maxStateOfCharge );
						var batteryLevelHeight = normalizedStateOfCharge * batteryLevelGauge.clip.height;

						batteryLevelGauge.clip.rect.setAttribute( "height", batteryLevelHeight );
						batteryLevelGauge.clip.rect.setAttribute( "y", batteryLevelGauge.clip.y + ( batteryLevelGauge.clip.height - batteryLevelHeight ) );

						if ( maxPowerDraw === 0 ) {
							powerDrawPosHeight = powerDrawNegHeight = 0;
						} else {
							var halfHeight = powerDrawGauge.clip.height / 2.0;

							if ( powerDrawAvg >= 0 ) {
								var powerDrawPosNormalized = isNaN( powerDrawAvg ) ? 0 : clamp( powerDrawAvg / maxPowerDraw, 0, 1 );
								var powerDrawHeight = Math.abs( powerDrawPosNormalized ) * halfHeight;

								powerDrawGauge.clip.rect.setAttribute( "y", powerDrawGauge.clip.y + ( halfHeight - powerDrawHeight ) );
								powerDrawGauge.clip.rect.setAttribute( "height", powerDrawHeight );
							} else {
								var powerDrawNegNormalized = isNaN( powerDrawAvg ) ? 0 : clamp( powerDrawAvg / maxPowerDraw, -1, 0 );
								var powerDrawHeight = Math.abs( powerDrawNegNormalized ) * halfHeight;

								powerDrawGauge.clip.rect.setAttribute( "y", powerDrawGauge.clip.y + halfHeight );
								powerDrawGauge.clip.rect.setAttribute( "height", powerDrawHeight );
							}
						}
					} );
				} );
			}
		};
	} ] );
